"use strict";
/*tslint:disable:no-shadowed-variable*/
/*tslint:disable:max-classes-per-file*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("mz/fs");
const fs = __importStar(require("mz/fs"));
const npm_registry_fetch_1 = __importDefault(require("npm-registry-fetch"));
const semver = __importStar(require("semver"));
const url_1 = require("url");
// import { throwIfCancelled } from './cancellation'
// import { Logger } from './logging'
const resources_1 = require("./resources");
class Span {
}
class Tracer {
}
function tracePromise(operationName, tracer, childOf, operation) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield operation({});
    });
}
function logErrorEvent(span, err) {
    // span.log({ event: ERROR, 'error.object': err, stack: err.stack, message: err.message })
}
exports.logErrorEvent = logErrorEvent;
function fetchPackageMeta(packageName, versionSpec = 'latest', npmConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = Object.assign({}, npmConfig, { spec: packageName });
        if (!packageName.startsWith('@') && (versionSpec === 'latest' || semver.valid(versionSpec))) {
            // Request precise version
            const result = yield npm_registry_fetch_1.default.json(`/${packageName}/${versionSpec}`, options);
            return result;
        }
        // Resolve version
        const result = yield npm_registry_fetch_1.default.json(`/${packageName}`, options);
        if (result.versions[result['dist-tags'][versionSpec]]) {
            return result.versions[result['dist-tags'][versionSpec]];
        }
        const versions = Object.keys(result.versions);
        const version = semver.maxSatisfying(versions, versionSpec);
        if (!version) {
            throw new Error(`Version ${packageName}@${versionSpec} does not exist`);
        }
        return result.versions[version];
    });
}
exports.fetchPackageMeta = fetchPackageMeta;
/**
 * Checks if a dependency from a package.json should be installed or not by checking
 * whether it contains TypeScript typings.
 */
function hasTypes(name, range, npmConfig, tracer, span) {
    return tracePromise('Fetch package metadata', tracer, span, (span) => __awaiter(this, void 0, void 0, function* () {
        // span.setTag('name', name)
        const version = semver.validRange(range) || 'latest';
        // span.setTag('version', version)
        const dependencyPackageJson = yield fetchPackageMeta(name, version, npmConfig);
        // Keep packages only if they have a types or typings field
        return !!dependencyPackageJson.typings || !!dependencyPackageJson.types;
    }));
}
/**
 * Removes all dependencies from a package.json that do not contain TypeScript type declaration files.
 *
 * @param packageJsonPath File path to a package.json
 * @return Whether the package.json contained any dependencies
 */
function filterDependencies(packageJsonPath, { npmConfig, logger, tracer, span, token, }) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield tracePromise('Filter dependencies', tracer, span, (span) => __awaiter(this, void 0, void 0, function* () {
            // span.setTag('packageJsonPath', packageJsonPath)
            logger.log('Filtering package.json at ', packageJsonPath);
            const packageJson = JSON.parse(yield fs.readFile(packageJsonPath, 'utf-8'));
            const excluded = [];
            const included = [];
            yield Promise.all(['dependencies', 'devDependencies', 'optionalDependencies'].map((dependencyType) => __awaiter(this, void 0, void 0, function* () {
                const dependencies = packageJson[dependencyType];
                if (!dependencies) {
                    return;
                }
                yield Promise.all(
                // @ts-ignore
                Object.entries(dependencies).map(([name, range]) => __awaiter(this, void 0, void 0, function* () {
                    // throwIfCancelled(token)
                    try {
                        if (name.startsWith('@types/') || (yield hasTypes(name, range, npmConfig, tracer, span))) {
                            included.push(name);
                        }
                        else {
                            excluded.push(name);
                            dependencies[name] = undefined;
                        }
                    }
                    catch (err) {
                        // throwIfCancelled(token)
                        included.push(name);
                        logger.error(`Error inspecting dependency ${name}@${range} in ${packageJsonPath}`, err);
                        logErrorEvent(span, err);
                    }
                })));
            })));
            // span.setTag('excluded', excluded.length)
            // span.setTag('included', included.length)
            logger.log(`Excluding ${excluded.length} dependencies`);
            logger.log(`Keeping ${included.length} dependencies`);
            // Only write if there is any change to dependencies
            if (included.length > 0 && excluded.length > 0) {
                yield fs.writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
            }
            return included.length > 0;
        }));
    });
}
exports.filterDependencies = filterDependencies;
/**
 * Finds the closest package.json for a given URL.
 *
 * @param resource The URL from which to walk upwards.
 * @param rootUri A URL at which to stop searching. If not given, defaults to the root of the resource URL.
 */
function findClosestPackageJson(resource, rootUri = Object.assign(new url_1.URL(resource.href), { pathname: '' })) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const parent of resources_1.walkUp(resource)) {
            if (!parent.href.startsWith(rootUri.href)) {
                break;
            }
            const packageJsonUri = new url_1.URL('package.json', parent.href);
            // TODO why original package don't have this check?
            if (!fs.existsSync(url_1.fileURLToPath(packageJsonUri))) {
                continue;
            }
            try {
                const packageJson = yield readPackageJson(packageJsonUri);
                return [packageJsonUri, packageJson];
            }
            catch (err) {
                // TODO remove this
                if (err instanceof resources_1.ResourceNotFoundError) {
                    continue;
                }
                throw err;
            }
        }
        throw new Error(`No package.json found for ${resource} under root ${rootUri}`);
    });
}
exports.findClosestPackageJson = findClosestPackageJson;
exports.isDefinitelyTyped = (uri) => uri.pathname.includes('DefinitelyTyped/DefinitelyTyped');
/**
 * Finds the package name and package root that the given URI belongs to.
 * Handles special repositories like DefinitelyTyped.
 */
function findPackageRootAndName(uri, { span, tracer }) {
    return __awaiter(this, void 0, void 0, function* () {
        // Special case: if the definition is in DefinitelyTyped, the package name is @types/<subfolder>[/<version>]
        if (exports.isDefinitelyTyped(uri)) {
            const dtMatch = uri.pathname.match(/\/types\/([^\/]+)\//);
            if (dtMatch) {
                const packageRoot = new url_1.URL(uri.href);
                // Strip everything after types/ (except the optional version directory)
                packageRoot.pathname = packageRoot.pathname.replace(/\/types\/([^\/]+)\/(v[^\/]+\/)?.*$/, '/types/$1/$2');
                const packageName = '@types/' + dtMatch[1];
                return [packageRoot, packageName];
            }
        }
        // Find containing package
        const [packageJsonUrl, packageJson] = yield findClosestPackageJson(uri, undefined);
        if (!packageJson.name) {
            throw new Error(`package.json at ${packageJsonUrl} does not contain a name`);
        }
        const packageRoot = new url_1.URL('.', packageJsonUrl);
        return [packageRoot, packageJson.name];
    });
}
exports.findPackageRootAndName = findPackageRootAndName;
const packageCache = new Map();
function readPackageJson(pkgJsonUri) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = url_1.fileURLToPath(pkgJsonUri);
        if (packageCache.has(path)) {
            return packageCache.get(path);
        }
        else {
            const json = JSON.parse(yield fs_1.readFile(path, 'utf-8'));
            packageCache.set(path, json);
            return json;
        }
    });
}
exports.readPackageJson = readPackageJson;
function cloneUrlFromPackageMeta(packageMeta) {
    if (!packageMeta.repository) {
        throw new Error(`Package ${packageMeta.name} data does not contain repository field`);
    }
    let repoUrl = typeof packageMeta.repository === 'string' ? packageMeta.repository : packageMeta.repository.url;
    // GitHub shorthand
    if (/^[^\/]+\/[^\/]+$/.test(repoUrl)) {
        repoUrl = 'https://github.com/' + repoUrl;
    }
    return repoUrl;
}
exports.cloneUrlFromPackageMeta = cloneUrlFromPackageMeta;
/**
 * @param filePath e.g. `/foo/node_modules/pkg/dist/bar.ts`
 * @returns e.g. `foo/node_modules/pkg`
 */
function resolveDependencyRootDir(filePath) {
    const parts = filePath.split('/');
    while (parts.length > 0 &&
        !(parts[parts.length - 2] === 'node_modules' ||
            (parts[parts.length - 3] === 'node_modules' && parts[parts.length - 2].startsWith('@')))) {
        parts.pop();
    }
    return parts.join('/');
}
exports.resolveDependencyRootDir = resolveDependencyRootDir;
//# sourceMappingURL=dependencies.js.map