"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const extended_lsp_server_1 = require("./extended-lsp-server");
const logger_1 = require("@elastic/typescript-language-server/lib/logger");
const lsp_client_1 = require("@elastic/typescript-language-server/lib/lsp-client");
const lspcalls = __importStar(require("@elastic/typescript-language-server/lib/lsp-protocol.calls.proposed"));
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
const lsp = __importStar(require("vscode-languageserver"));
var FullRequest;
(function (FullRequest) {
    FullRequest.type = new vscode_jsonrpc_1.RequestType('textDocument/full');
})(FullRequest = exports.FullRequest || (exports.FullRequest = {}));
var EDefinitionRequest;
(function (EDefinitionRequest) {
    EDefinitionRequest.type = new vscode_jsonrpc_1.RequestType('textDocument/edefinition');
})(EDefinitionRequest = exports.EDefinitionRequest || (exports.EDefinitionRequest = {}));
function createLspConnection(options) {
    const connection = lsp.createConnection();
    const lspClient = new lsp_client_1.LspClientImpl(connection);
    const logger = new logger_1.LspClientLogger(lspClient, options.showMessageLevel);
    const server = new extended_lsp_server_1.ExtendedLspServer({
        logger,
        lspClient,
        tsserverPath: options.tsserverPath,
        tsserverLogFile: options.tsserverLogFile,
        tsserverLogVerbosity: options.tsserverLogVerbosity,
        otherOptions: options.otherOptions,
    });
    connection.onInitialize(server.initialize.bind(server));
    connection.onDidOpenTextDocument(server.didOpenTextDocument.bind(server));
    connection.onDidSaveTextDocument(server.didSaveTextDocument.bind(server));
    connection.onDidCloseTextDocument(server.didCloseTextDocument.bind(server));
    connection.onDidChangeTextDocument(server.didChangeTextDocument.bind(server));
    connection.onCodeAction(server.codeAction.bind(server));
    connection.onCompletion(server.completion.bind(server));
    connection.onCompletionResolve(server.completionResolve.bind(server));
    connection.onDefinition(server.definition.bind(server));
    connection.onImplementation(server.implementation.bind(server));
    connection.onTypeDefinition(server.typeDefinition.bind(server));
    connection.onDocumentFormatting(server.documentFormatting.bind(server));
    connection.onDocumentHighlight(server.documentHighlight.bind(server));
    connection.onDocumentSymbol(server.documentSymbol.bind(server));
    connection.onExecuteCommand(server.executeCommand.bind(server));
    connection.onHover(server.hover.bind(server));
    connection.onReferences(server.references.bind(server));
    connection.onRenameRequest(server.rename.bind(server));
    connection.onSignatureHelp(server.signatureHelp.bind(server));
    connection.onWorkspaceSymbol(server.workspaceSymbol.bind(server));
    connection.onFoldingRanges(server.foldingRanges.bind(server));
    connection.onExit(server.exit.bind(server));
    // proposed `textDocument/calls` request
    connection.onRequest(lspcalls.CallsRequest.type, server.calls.bind(server));
    // Add our extension
    connection.onRequest(FullRequest.type, server.full.bind(server));
    connection.onRequest(EDefinitionRequest.type, server.edefinition.bind(server));
    return connection;
}
exports.createLspConnection = createLspConnection;
//# sourceMappingURL=lsp-connection.js.map